@@ -107,6 +107,7 @@ class _GenericAdvFuncHelper:
     backend_name = None # overriden in subclasses ('postgres', 'sqlite', etc.)
     needs_from_clause = False
     union_parentheses_support = True
+    intersect_all_support = True
     users_support = True
     groups_support = True
     ilike_support = True
@@ -384,12 +385,8 @@ class _PGAdvFuncHelper(_GenericAdvFuncHelper):
             cursor.execute('CREATE LANGUAGE %s' % extlang)
             print '%s language installed' % extlang
 
-    def list_users(self, cursor, username=None):
+    def list_users(self, cursor):
         """return the list of existing database users"""
-        if username:
-            warn('username argument is deprecated, use user_exists method',
-                 DeprecationWarning, stacklevel=2)
-            return self.user_exists(cursor, username)
         cursor.execute("SELECT usename FROM pg_user")
         return [r[0] for r in cursor.fetchall()]
 
@@ -426,6 +423,7 @@ class _SqliteAdvFuncHelper(_GenericAdvFuncHelper):
     users_support = groups_support = False
     ilike_support = False
     union_parentheses_support = False
+    intersect_all_support = False
     
     def sql_create_index(self, table, column, unique=False):
         idx = self._index_name(table, column, unique)
@@ -465,6 +463,8 @@ class _MyAdvFuncHelper(_GenericAdvFuncHelper):
     TYPE_MAPPING['Password'] = 'tinyblob'
     TYPE_MAPPING['String'] = 'mediumtext'
     TYPE_MAPPING['Bytes'] = 'longblob'
+    # don't use timestamp which is automatically updated on row update
+    TYPE_MAPPING['Datetime'] = 'datetime'
     
     def system_database(self):
         """return the system database for the given driver"""
