@@ -1,75 +1,94 @@
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""Some classes used to handle advanced configuration in simple to
-complex applications.
+"""Classes to handle advanced configuration in simple to complex applications.
 
-It's able to load the configuration from a file and or command line
-options, to generate a sample configuration file or to display program's
-usage. It basically fill the gap between optik/optparse and ConfigParser,
-with some additional data types (available as standalone optik extension
-in the `optik_ext` module)
+Allows to load the configuration from a file or from command line
+options, to generate a sample configuration file or to display
+program's usage. Fills the gap between optik/optparse and ConfigParser
+by adding data types (which are also available as a standalone optik
+extension in the `optik_ext` module).
 
 
 Quick start: simplest usage
-```````````````````````````
+---------------------------
 
-import sys
-from logilab.common.configuration import Configuration
+.. python ::
 
-options = [('dothis', {'type':'yn', 'default': True, 'metavar': '<y or n>'}),
-           ('value', {'type': 'string', 'metavar': '<string>'}),
-           ('multiple', {'type': 'csv', 'default': ('yop',),
-                         'metavar': '<comma separated values>',
-                         'help': 'you can also document the option'}),
-           ('number', {'type': 'int', 'default':2, 'metavar':'<int>'}),
-           ]
-config = Configuration(options=options, name='My config')
-print config['dothis']
-print config['value']
-print config['multiple']
-print config['number']
-
-print config.help()
-
-f = open('myconfig.ini', 'w')
-f.write('''[MY CONFIG]
-number = 3
-dothis = no
-multiple = 1,2,3
-''')
-f.close()
-config.load_file_configuration('myconfig.ini')
-print config['dothis']
-print config['value']
-print config['multiple']
-print config['number']
-
-sys.argv = ['mon prog', '--value', 'bacon', '--multiple', '4,5,6',
-            'nonoptionargument']
-print config.load_command_line_configuration()
-print config['value']
-
-config.generate_config()
+  >>> import sys
+  >>> from logilab.common.configuration import Configuration
+  >>> options = [('dothis', {'type':'yn', 'default': True, 'metavar': '<y or n>'}),
+  ...            ('value', {'type': 'string', 'metavar': '<string>'}),
+  ...            ('multiple', {'type': 'csv', 'default': ('yop',),
+  ...                          'metavar': '<comma separated values>',
+  ...                          'help': 'you can also document the option'}),
+  ...            ('number', {'type': 'int', 'default':2, 'metavar':'<int>'}),
+  ...           ]
+  >>> config = Configuration(options=options, name='My config')
+  >>> print config['dothis']
+  True
+  >>> print config['value']
+  None
+  >>> print config['multiple']
+  ('yop',)
+  >>> print config['number']
+  2
+  >>> print config.help()
+  Usage:  [options]
+  
+  Options:
+    -h, --help            show this help message and exit
+    --dothis=<y or n>     
+    --value=<string>      
+    --multiple=<comma separated values>
+                          you can also document the option [current: none]
+    --number=<int>        
+  
+  >>> f = open('myconfig.ini', 'w')
+  >>> f.write('''[MY CONFIG]
+  ... number = 3
+  ... dothis = no
+  ... multiple = 1,2,3
+  ... ''')
+  >>> f.close()
+  >>> config.load_file_configuration('myconfig.ini')
+  >>> print config['dothis']
+  False
+  >>> print config['value']
+  None
+  >>> print config['multiple']
+  ['1', '2', '3']
+  >>> print config['number']
+  3
+  >>> sys.argv = ['mon prog', '--value', 'bacon', '--multiple', '4,5,6',
+  ...             'nonoptionargument']
+  >>> print config.load_command_line_configuration()
+  ['nonoptionargument']
+  >>> print config['value']
+  bacon
+  >>> config.generate_config()
+  # class for simple configurations which don't need the
+  # manager / providers model and prefer delegation to inheritance
+  # 
+  # configuration values are accessible through a dict like interface
+  # 
+  [MY CONFIG]
+  
+  dothis=no
+  
+  value=bacon
+  
+  # you can also document the option
+  multiple=4,5,6
+  
+  number=3
+  >>>
 
 
-:author:    Logilab
-:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
+:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 """
-
 from __future__ import generators 
-
 __docformat__ = "restructuredtext en"
+
 __all__ = ('OptionsManagerMixIn', 'OptionsProviderMixIn',
            'ConfigurationMixIn', 'Configuration',
            'OptionsManager2ConfigurationAdapter')
@@ -77,7 +96,7 @@ __all__ = ('OptionsManagerMixIn', 'OptionsProviderMixIn',
 import os
 import sys
 import re
-from os.path import exists
+from os.path import exists, expanduser
 from copy import copy
 from ConfigParser import ConfigParser, NoOptionError, NoSectionError
 
@@ -93,6 +112,20 @@ REQUIRED = []
 class UnsupportedAction(Exception):
     """raised by set_option when it doesn't know what to do for an action"""
 
+
+def _get_encoding(encoding, stream):
+    encoding = encoding or getattr(stream, 'encoding', None)
+    if not encoding:
+        import locale
+        encoding = locale.getpreferredencoding()
+    return encoding
+
+def _encode(string, encoding):
+    if isinstance(string, unicode):
+        return string.encode(encoding)
+    return str(string)
+
+
 # validation functions ########################################################
 
 def choice_validator(opt_dict, name, value):
@@ -143,7 +176,7 @@ def password_validator(opt_dict, name, value):
 
 def date_validator(opt_dict, name, value):
     """validate and return a mx DateTime object for option of type 'date'"""
-    return check_password(None, name, value)
+    return check_date(None, name, value)
 
 
 VALIDATORS = {'string' : unquote,
@@ -164,7 +197,7 @@ VALIDATORS = {'string' : unquote,
               }
 
 def _call_validator(opttype, optdict, option, value):
-    if not VALIDATORS.has_key(opttype):
+    if opttype not in VALIDATORS:
         raise Exception('Unsupported type "%s"' % opttype)
     try:
         return VALIDATORS[opttype](optdict, option, value)
@@ -267,40 +300,48 @@ def format_option_value(optdict, value):
         value = "'%s'" % value
     return value
 
-def ini_format_section(stream, section, options, doc=None):
+def ini_format_section(stream, section, options, encoding=None, doc=None):
     """format an options section using the INI format"""
+    encoding = _get_encoding(encoding, stream)
     if doc:
-        print >> stream, comment(doc)
+        print >> stream, _encode(comment(doc), encoding)
     print >> stream, '[%s]' % section
     for optname, optdict, value in options:
         value = format_option_value(optdict, value)
         help = optdict.get('help')
         if help:
+            help = normalize_text(help, line_len=79, indent='# ')
             print >> stream
-            print >> stream, normalize_text(help, line_len=79, indent='# ')
+            print >> stream, _encode(help, encoding)
         else:
             print >> stream
         if value is None:
             print >> stream, '#%s=' % optname
         else:
-            print >> stream, '%s=%s' % (optname, str(value).strip())
+            value = _encode(value, encoding).strip()
+            print >> stream, '%s=%s' % (optname, value)
         
 format_section = ini_format_section
 
-def rest_format_section(stream, section, options, doc=None):
+def rest_format_section(stream, section, options, encoding=None, doc=None):
     """format an options section using the INI format"""
+    encoding = _get_encoding(encoding, stream)
     if section:
         print >> stream, '%s\n%s' % (section, "'"*len(section))
     if doc:
-        print >> stream, normalize_text(doc, line_len=79, indent='')
+        print >> stream, _encode(normalize_text(doc, line_len=79, indent=''),
+                                 encoding)
         print >> stream
     for optname, optdict, value in options:
         help = optdict.get('help')
         print >> stream, ':%s:' % optname
         if help:
-            print >> stream, normalize_text(help, line_len=79, indent='  ')
+            help = normalize_text(help, line_len=79, indent='  ')
+            print >> stream, _encode(help, encoding)
         if value:
-            print >> stream, '  Default: %s' % format_option_value(optdict, value)
+            value = _encode(format_option_value(optdict, value), encoding)
+            print >> stream, ''
+            print >> stream, '  Default: ``%s``' % value.replace("`` ","```` ``")
 
 
 class OptionsManagerMixIn(object):
@@ -337,26 +378,29 @@ class OptionsManagerMixIn(object):
         else:
             self.options_providers.append(provider)
         non_group_spec_options = [option for option in provider.options
-                                  if not option[1].has_key('group')]
+                                  if 'group' not in option[1]]
         groups = getattr(provider, 'option_groups', ())
         if own_group:
             self.add_option_group(provider.name.upper(), provider.__doc__,
                                   non_group_spec_options, provider)
         else:
             for opt_name, opt_dict in non_group_spec_options:
-                args, opt_dict = self.optik_option(provider, opt_name, opt_dict)
+                args, opt_dict = self.optik_option(provider, opt_name, opt_dict)                
                 self._optik_parser.add_option(*args, **opt_dict)
-                self._all_options[opt_name] = provider                
+                
+                self._all_options[opt_name] = provider
         for gname, gdoc in groups:
+            gname = gname.upper()
             goptions = [option for option in provider.options
-                        if option[1].get('group') == gname]
+                        if option[1].get('group', '').upper() == gname]
             self.add_option_group(gname, gdoc, goptions, provider)
         
     def add_option_group(self, group_name, doc, options, provider):
         """add an option group including the listed options
         """
         # add section to the config file
-        self._config_parser.add_section(group_name)
+        if group_name != "DEFAULT":
+            self._config_parser.add_section(group_name)
         # add option group to the command line parser
         if options:
             group = OptionGroup(self._optik_parser,
@@ -373,23 +417,24 @@ class OptionsManagerMixIn(object):
         use with optik/optparse
         """
         opt_dict = copy(opt_dict)
-        if opt_dict.has_key('action'):
+        if 'action' in opt_dict:
             self._nocallback_options[provider] = opt_name
         else:
             opt_dict['action'] = 'callback'
             opt_dict['callback'] = self.cb_set_provider_option
-        for specific in ('default', 'group', 'inputlevel'):
-            if opt_dict.has_key(specific):
-                del opt_dict[specific]
-                if (OPTPARSE_FORMAT_DEFAULT
-                    and specific == 'default' and opt_dict.has_key('help')):
-                    opt_dict['help'] += ' [current: %default]'
+        # default is handled here and *must not* be given to optik if you
+        # want the whole machinery to work
+        if 'default' in opt_dict:
+            if OPTPARSE_FORMAT_DEFAULT and 'help' in opt_dict:
+                opt_dict['help'] += ' [current: %default]'
+            del opt_dict['default']
         args = ['--' + opt_name]
-        if opt_dict.has_key('short'):
+        if 'short' in opt_dict:
             self._short_options[opt_dict['short']] = opt_name
             args.append('-' + opt_dict['short'])
             del opt_dict['short']
         available_keys = set(self._optik_parser.option_class.ATTRS)
+        # cleanup option definition dict before giving it to optik
         for key in opt_dict.keys():
             if not key in available_keys:
                 opt_dict.pop(key)
@@ -412,11 +457,12 @@ class OptionsManagerMixIn(object):
         """set option on the correct option provider"""
         self._all_options[opt_name].set_option(opt_name, value)
 
-    def generate_config(self, stream=None, skipsections=()):
+    def generate_config(self, stream=None, skipsections=(), encoding=None):
         """write a configuration file according to the current configuration
         into the given stream or stdout
         """
         stream = stream or sys.stdout
+        encoding = _get_encoding(encoding, stream)
         printed = False
         for provider in self.options_providers:
             default_options = []
@@ -435,15 +481,19 @@ class OptionsManagerMixIn(object):
                     doc = None
                 if printed:
                     print >> stream, '\n'
-                format_section(stream, section.upper(), options, doc)
+                format_section(stream, section.upper(), options, encoding, doc)
                 printed = True
 
     def generate_manpage(self, pkginfo, section=1, stream=None):
         """write a man page for the current configuration into the given
         stream or stdout
         """
-        generate_manpage(self._optik_parser, pkginfo,
-                         section, stream=stream or sys.stdout)
+        self._monkeypatch_expand_default()
+        try:
+            generate_manpage(self._optik_parser, pkginfo,
+                             section, stream=stream or sys.stdout)
+        finally:
+            self._unmonkeypatch_expand_default()
         
     # initialization methods ##################################################
 
@@ -463,8 +513,15 @@ class OptionsManagerMixIn(object):
         """
         if config_file is None:
             config_file = self.config_file
+        if config_file is not None:
+            config_file = expanduser(config_file)
         if config_file and exists(config_file):
-            self._config_parser.read([config_file])
+            parser = self._config_parser
+            parser.read([config_file])
+            # normalize sections'title
+            for sect, values in parser._sections.items():
+                if not sect.isupper() and values:
+                    parser._sections[sect.upper()] = values
         elif not self.quiet:
             msg = 'No config file found, using default configuration'
             print >> sys.stderr, msg
@@ -511,13 +568,7 @@ class OptionsManagerMixIn(object):
 
         return additional arguments
         """
-        # monkey patch optparse to deal with our default values
-        try:
-            expand_default_backup = HelpFormatter.expand_default
-            HelpFormatter.expand_default = expand_default
-        except AttributeError:
-            # python < 2.4: nothing to be done
-            pass
+        self._monkeypatch_expand_default()
         try:
             if args is None:
                 args = sys.argv[1:]
@@ -533,9 +584,7 @@ class OptionsManagerMixIn(object):
                     setattr(config, attr, value)
             return args
         finally:
-            if hasattr(HelpFormatter, 'expand_default'):
-                # unpatch optparse to avoid side effects
-                HelpFormatter.expand_default = expand_default_backup
+            self._unmonkeypatch_expand_default()
 
 
     # help methods ############################################################
@@ -546,10 +595,28 @@ class OptionsManagerMixIn(object):
                             title=title.capitalize(),
                             description=description)
         self._optik_parser.add_option_group(group)
+
+    def _monkeypatch_expand_default(self):
+        # monkey patch optparse to deal with our default values
+        try:
+            self.__expand_default_backup = HelpFormatter.expand_default
+            HelpFormatter.expand_default = expand_default
+        except AttributeError:
+            # python < 2.4: nothing to be done
+            pass
+    def _unmonkeypatch_expand_default(self):
+        # remove monkey patch
+        if hasattr(HelpFormatter, 'expand_default'):
+            # unpatch optparse to avoid side effects
+            HelpFormatter.expand_default = self.__expand_default_backup
         
     def help(self):
-        """return the usage string for available options """
-        return self._optik_parser.format_help()
+        """return the usage string for available options """ 
+        self._monkeypatch_expand_default()
+        try:
+            return self._optik_parser.format_help()
+        finally:
+            self._unmonkeypatch_expand_default()
     
 
 class Method(object):
@@ -569,7 +636,7 @@ class Method(object):
         assert self._inst, 'unbound method'
         return getattr(self._inst, self.method)()
 
-        
+
 class OptionsProviderMixIn(object):
     """Mixin to provide options to an OptionsManager"""
     
@@ -656,6 +723,8 @@ class OptionsProviderMixIn(object):
                 setattr(self.config, opt_name, _list + (value,))
             else:
                 _list.append(value)
+        elif action == 'callback':
+            opt_dict['callback'](None, opt_name, value, None)
         else:
             raise UnsupportedAction(action)
 
@@ -730,7 +799,7 @@ class ConfigurationMixIn(OptionsManagerMixIn, OptionsProviderMixIn):
             self.option_groups = []
             for option, optdict in self.options:
                 try:
-                    gdef = (optdict['group'], '')
+                    gdef = (optdict['group'].upper(), '')
                 except KeyError:
                     continue
                 if not gdef in self.option_groups:
@@ -749,6 +818,9 @@ class ConfigurationMixIn(OptionsManagerMixIn, OptionsProviderMixIn):
     def load_defaults(self):
         OptionsProviderMixIn.load_defaults(self)
 
+    def __iter__(self):
+        return iter(self.config.__dict__.iteritems())
+    
     def __getitem__(self, key):
         try:
             return getattr(self.config, self.option_name(key))
