@@ -1,27 +1,19 @@
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""this module contains some function/method decorators
+"""A few useful function/method decorators.
 
-:author:    Logilab
-:copyright: 2006-2008 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
+:copyright: 2006-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 """
 __docformat__ = "restructuredtext en"
 
+from types import MethodType
+from time import clock
+import sys, re
+
 # XXX rewrite so we can use the decorator syntax when keyarg has to be specified
 
 def cached(callableobj, keyarg=None):
-    """simple decorator to cache result of method call"""
+    """Simple decorator to cache result of method call."""
     #print callableobj, keyarg, callableobj.func_code.co_argcount
     if callableobj.func_code.co_argcount == 1 or keyarg == 0:
         
@@ -75,14 +67,14 @@ def cached(callableobj, keyarg=None):
     return cache_wrapper3
 
 def clear_cache(obj, funcname):
-    """function to clear a cache handled by the cached decorator"""
+    """Function to clear a cache handled by the cached decorator."""
     try:
         del obj.__dict__['_%s_cache_' % funcname]
     except KeyError:
         pass
 
 def copy_cache(obj, funcname, cacheobj):
-    """copy cache for <funcname> from cacheobj to obj"""
+    """Copy cache for <funcname> from cacheobj to obj."""
     cache = '_%s_cache_' % funcname
     try:
         setattr(obj, cache, cacheobj.__dict__[cache])
@@ -91,8 +83,8 @@ def copy_cache(obj, funcname, cacheobj):
 
 
 class wproperty(object):
-    """simple descriptor expecting to take a modifier function as first argument
-    and looking for a _<function name> to retreive the attribute
+    """Simple descriptor expecting to take a modifier function as first argument
+    and looking for a _<function name> to retrieve the attribute.
     """
     def __init__(self, setfunc):
         self.setfunc = setfunc
@@ -112,7 +104,21 @@ class classproperty(object):
     def __get__(self, inst, cls):
         return self.get(cls)
 
-from time import clock
+
+class iclassmethod(object):
+    '''Descriptor for method which should be available as class method if called
+    on the class or instance method if called on an instance.
+    '''
+    def __init__(self, func):
+        self.func = func
+    def __get__(self, instance, objtype):
+        if instance is None:
+            return MethodType(self.func, objtype, objtype.__class__)
+        return MethodType(self.func, instance, objtype)
+    def __set__(self, instance, value):
+        raise AttributeError("can't set attribute")
+
+
 def timed(f):
     def wrap(*args, **kwargs):
         t = clock()
@@ -122,3 +128,42 @@ def timed(f):
         return res
     return wrap
 
+
+def locked(acquire, release):
+    """Decorator taking two methods to acquire/release a lock as argument,
+    returning a decorator function which will call the inner method after
+    having called acquire(self) et will call release(self) afterwards.
+    """
+    def decorator(f):
+        def wrapper(self, *args, **kwargs):
+            acquire(self)
+            try:
+                return f(self, *args, **kwargs)
+            finally:
+                release(self)
+        return wrapper
+    return decorator
+
+
+def monkeypatch(klass, methodname=None):
+    """Decorator extending class with the decorated function
+    >>> class A:
+    ...     pass
+    >>> @monkeypatch(A)
+    ... def meth(self):
+    ...     return 12
+    ...
+    >>> a = A()
+    >>> a.meth()
+    12
+    >>> @monkeypatch(A, 'foo')
+    ... def meth(self):
+    ...     return 12
+    ...
+    >>> a.foo()
+    12
+    """
+    def decorator(func):
+        setattr(klass, methodname or func.__name__, func)
+        return func
+    return decorator
