@@ -1,26 +1,10 @@
-# -*- coding: iso-8859-1 -*-
-# Copyright (c) 2003-2008 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# -*- coding: utf-8 -*-
 """Python modules manipulation utility functions.
 
-:author:    Logilab
-:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-
-
+:author: Logilab
+:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 
 :type PY_SOURCE_EXTS: tuple(str)
 :var PY_SOURCE_EXTS: list of possible python source file extension
@@ -31,7 +15,6 @@
 :type BUILTIN_MODULES: dict
 :var BUILTIN_MODULES: dictionary with builtin module names has key
 """
-
 __docformat__ = "restructuredtext en"
 
 import sys
@@ -39,6 +22,13 @@ import os
 from os.path import walk, splitext, join, abspath, isdir, dirname, exists
 from imp import find_module, load_module, C_BUILTIN, PY_COMPILED, PKG_DIRECTORY
 
+try:
+    import zipimport
+except ImportError:
+    zipimport = None
+    
+ZIPFILE = object()
+
 from logilab.common import STD_BLACKLIST
 
 if sys.platform.startswith('win'):
@@ -82,7 +72,7 @@ class LazyObject(object):
 
 
 def load_module_from_name(dotted_name, path=None, use_sys=1):
-    """load a Python module from it's name
+    """Load a Python module from it's name.
 
     :type dotted_name: str
     :param dotted_name: python name of a module or package
@@ -107,7 +97,7 @@ def load_module_from_name(dotted_name, path=None, use_sys=1):
 
 
 def load_module_from_modpath(parts, path=None, use_sys=1):
-    """load a python module from it's splitted name
+    """Load a python module from it's splitted name.
 
     :type parts: list(str) or tuple(str)
     :param parts:
@@ -122,9 +112,6 @@ def load_module_from_modpath(parts, path=None, use_sys=1):
     :param use_sys:
       boolean indicating whether the sys.modules dictionary should be used or not
 
-    :param _prefix: used internally, should not be specified
-
-
     :raise ImportError: if the module or package is not found
     
     :rtype: module
@@ -158,10 +145,10 @@ def load_module_from_modpath(parts, path=None, use_sys=1):
 
 
 def load_module_from_file(filepath, path=None, use_sys=1):
-    """load a Python module from it's path
+    """Load a Python module from it's path.
 
     :type filepath: str
-    :param dotted_name: path to the python module or package
+    :param filepath: path to the python module or package
 
     :type path: list or None
     :param path:
@@ -547,6 +534,16 @@ def _file_from_modpath(modpath, path=None, context=None):
         mp_filename = _has_init(mp_filename)
     return mp_filename
 
+def _search_zip(modpath, pic):
+    for filepath, importer in pic.items():
+        if importer is not None:
+            if importer.find_module(modpath[0]):
+                if not importer.find_module('/'.join(modpath)):
+                    raise ImportError('No module named %s in %s/%s' % (
+                        '.'.join(modpath[1:]), file, modpath))
+                return ZIPFILE, abspath(filepath) + '/' + '/'.join(modpath), filepath
+    raise ImportError('No module named %s' % '.'.join(modpath))
+    
 def _module_file(modpath, path=None):
     """get a module type / file path
 
@@ -564,13 +561,48 @@ def _module_file(modpath, path=None):
     :rtype: tuple(int, str)
     :return: the module type flag and the file path for a module
     """
+    # egg support compat
+    try:
+        pic = sys.path_importer_cache
+        _path = (path is None and sys.path or path)
+        for __path in _path:
+            if not __path in pic:
+                try:
+                    pic[__path] = zipimport.zipimporter(__path)
+                except zipimport.ZipImportError:
+                    pic[__path] = None
+        checkeggs = True
+    except AttributeError:
+        checkeggs = False
+    imported = []
     while modpath:
-        _, mp_filename, mp_desc = find_module(modpath[0], path)
-        modpath.pop(0)
+        try:
+            _, mp_filename, mp_desc = find_module(modpath[0], path)
+        except ImportError:
+            if checkeggs:
+                return _search_zip(modpath, pic)[:2]
+            raise
+        else:
+            if checkeggs:
+                fullabspath = [abspath(x) for x in _path]
+                try:
+                    pathindex = fullabspath.index(dirname(abspath(mp_filename)))
+                    emtype, emp_filename, zippath = _search_zip(modpath, pic)
+                    if pathindex > _path.index(zippath):
+                        # an egg takes priority
+                        return emtype, emp_filename
+                except ValueError:
+                    # XXX not in _path
+                    pass
+                except ImportError:
+                    pass
+                checkeggs = False
+        imported.append(modpath.pop(0))
         mtype = mp_desc[2]
         if modpath:
             if mtype != PKG_DIRECTORY:
-                raise ImportError('No module %r' % '.'.join(modpath))
+                raise ImportError('No module %s in %s' % ('.'.join(modpath),
+                                                          '.'.join(imported)))
             path = [mp_filename]
     return mtype, mp_filename
 
@@ -590,7 +622,7 @@ def _has_init(directory):
     else return None
     """
     mod_or_pack = join(directory, '__init__')
-    for ext in ('.py', '.pyw', '.pyc', '.pyo'):
-        if exists(mod_or_pack + ext):
+    for ext in PY_SOURCE_EXTS + ('.pyc', '.pyo'):
+        if exists(mod_or_pack + '.' + ext):
             return mod_or_pack + ext
     return None
