@@ -11,7 +11,23 @@ It also defines three new types for optik/optparse command line parser :
     argument of this type will be true if 'y' or 'yes', false if 'n' or 'no'
   * named
     argument of this type are in the form <NAME>=<VALUE> or <NAME>:<VALUE>
-
+  * password
+    argument of this type wont be converted but this is used by other tools
+    such as interactive prompt for configuration to double check value and
+    use an invisible field
+  * multiple_choice
+    same as default "choice" type but multiple choices allowed
+  * file
+    argument of this type wont be converted but checked that the given file exists
+  * color
+    argument of this type wont be converted but checked its either a
+    named color or a color specified using hexadecimal notation (preceded by a #)
+  * time
+    argument of this type will be converted to a float value in seconds
+    according to time units (ms, s, min, h, d)
+  * bytes
+    argument of this type will be converted to a float value in bytes
+    according to byte units (b, kb, mb, gb, tb)
 
 :copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
@@ -38,6 +54,11 @@ except ImportError:
         from optik import NO_DEFAULT
     except:
         NO_DEFAULT = []
+    # XXX check SUPPRESS_HELP availability
+    try:
+        from optik import SUPPRESS_HELP
+    except:
+        SUPPRESS_HELP = None
 
 try:
     from mx import DateTime
@@ -146,13 +167,22 @@ def check_color(option, opt, value):
     value or predefinied color"
     raise OptionValueError(msg % (opt, value))
 
+def check_time(option, opt, value):
+    from logilab.common.textutils import TIME_UNITS, apply_units
+    apply_units(value, TIME_UNITS)
+
+def check_bytes(option, opt, value):
+    from logilab.common.textutils import BYTE_UNITS, apply_units
+    apply_units(value, BYTE_UNITS)
+    
 import types
 
 class Option(BaseOption):
     """override optik.Option to add some new option types
     """
     TYPES = BaseOption.TYPES + ('regexp', 'csv', 'yn', 'named', 'password',
-                                'multiple_choice', 'file', 'font', 'color')
+                                'multiple_choice', 'file', 'color',
+                                'time', 'bytes')
     ATTRS = BaseOption.ATTRS + ['hide']
     TYPE_CHECKER = copy(BaseOption.TYPE_CHECKER)
     TYPE_CHECKER['regexp'] = check_regexp
@@ -163,6 +193,8 @@ class Option(BaseOption):
     TYPE_CHECKER['file'] = check_file
     TYPE_CHECKER['color'] = check_color
     TYPE_CHECKER['password'] = check_password
+    TYPE_CHECKER['time'] = check_time
+    TYPE_CHECKER['bytes'] = check_bytes
     if HAS_MX_DATETIME:
         TYPES += ('date',)
         TYPE_CHECKER['date'] = check_date
