@@ -1,7 +1,10 @@
-"""Table management module
-"""
+"""Table management module.
 
-__revision__ = '$Id: table.py,v 1.18 2006-04-09 22:30:53 nico Exp $'
+:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
+"""
+__docformat__ = "restructuredtext en"
 
 from warnings import warn
 
@@ -847,7 +850,7 @@ class DocbookRenderer(TableCellRenderer):
     def _render_cell_content(self, cell_content, table_style, col_index):
         """Makes the appropriate rendering for this cell content.
         Rendering properties will be searched using the
-        *table_style.get_xxx_by_index(col_index)' methods.
+        table_style.get_xxx_by_index(col_index)' methods.
         """
         try:
             align_on = self.properties['alignment']
