@@ -1,28 +1,15 @@
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """Some text manipulation utility functions.
 
 :author:    Logilab
-:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
+:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 
 :group text formatting: normalize_text, normalize_paragraph, pretty_match,\
 unquote, colorize_ansi
 :group text manipulation: searchall, get_csv
 :sort: text formatting, text manipulation
 
-
-
 :type ANSI_STYLES: dict(str)
 :var ANSI_STYLES: dictionary mapping style identifier to ANSI terminal code
 
@@ -41,12 +28,14 @@ unquote, colorize_ansi
 :var ANSI_RESET:
   ANSI terminal code reseting format defined by a previous ANSI escape sequence
 """
-
 __docformat__ = "restructuredtext en"
 
 import re
 from unicodedata import normalize as _uninormalize
-from os import linesep
+try:
+    from os import linesep
+except ImportError:
+    linesep = '\n' # gae
 
 
 MANUAL_UNICODE_MAP = {
@@ -242,6 +231,65 @@ def get_csv(string, sep=','):
     """
     return [word.strip() for word in string.split(sep) if word.strip()]
 
+_BLANK_URE = r'(\s|,)+'
+_BLANK_RE = re.compile(_BLANK_URE)
+__VALUE_URE = r'-?(([0-9]+\.[0-9]*)|((0x?)?[0-9]+))'
+__UNITS_URE = r'[a-zA-Z]+'
+_VALUE_RE = re.compile(r'(?P<value>%s)(?P<unit>%s)?'%(__VALUE_URE,__UNITS_URE))
+
+BYTE_UNITS = {
+     "B": 1,
+    "KB": 1024,
+    "MB": 1024 ** 2,
+    "GB": 1024 ** 3,
+    "TB": 1024 ** 4,
+}
+
+TIME_UNITS = {
+    "ms": 0.0001,
+    "s": 1,
+    "min": 60,
+    "h": 60 * 60,
+    "d": 60 * 60 *24,
+}
+
+def apply_units( string, units, inter=None, final=float, blank_reg=_BLANK_RE,
+    value_reg=_VALUE_RE):
+    """Parse the string applying the units defined in units
+    (eg: "1.5m",{'m',60} -> 80).
+        
+    :type string: str or unicode
+    :param string: the string to parse
+
+    :type units: dict (or any object with __getitem__ using basestring key)
+    :param units: a dict mapping a unit string repr to its value
+
+    :type inter: type
+    :param inter: used to parse every intermediate value (need __sum__)
+    
+    :type blank_reg: regexp
+    :param blank_reg: should match eveyr blank char to ignore.
+    
+    :type value_reg: regexp with "value" and optional "unit" group
+    :param value_reg: match a value and it's unit into the 
+    """
+    if inter is None:
+        inter = final
+
+    
+    string = _BLANK_RE.sub('',string)
+    values = []
+    for match in value_reg.finditer(string):
+        dic = match.groupdict()
+        #import sys
+        #print >> sys.stderr, dic
+        lit, unit = dic["value"], dic.get("unit")
+        value = inter(lit)
+        if unit is not None:
+            value *= units[unit]
+        values.append(value)
+
+    return final(sum(values))
 
 _LINE_RGX = re.compile('\r\n|\r+|\n')
 
